"use client";

import axios from "axios";
import * as React from 'react';
import { Calendar, Check, ChevronsUpDown, MoreHorizontal, Plus, Tags, Trash, User } from "lucide-react"
import { ApiKey, TtsUsernameFilter, TwitchConnection } from "@prisma/client";
import { useEffect, useState } from "react";
import { useSession } from "next-auth/react";
import Link from "next/link";
import { cn } from "@/lib/utils";
import { Skeleton } from "@/components/ui/skeleton";
import { Button } from "@/components/ui/button"
import { Command, CommandEmpty, CommandGroup, CommandInput, CommandItem, CommandList } from "@/components/ui/command"
import { Popover, PopoverContent, PopoverTrigger } from "@/components/ui/popover"
import { Table, TableBody, TableCaption, TableCell, TableHead, TableHeader, TableRow } from "@/components/ui/table";
import { Input } from "@/components/ui/input";
import { Textarea } from "@/components/ui/textarea";
import { useRouter } from "next/navigation";
import { useForm } from "react-hook-form";
import { Form, FormControl, FormField, FormItem, FormLabel, FormMessage } from "@/components/ui/form";
import * as z from "zod";
import { zodResolver } from "@hookform/resolvers/zod";
import { DropdownMenu, DropdownMenuGroup, DropdownMenuItem, DropdownMenuLabel, DropdownMenuSeparator, DropdownMenuShortcut, DropdownMenuSub, DropdownMenuSubContent, DropdownMenuSubTrigger } from "@/components/ui/dropdown-menu";
import { DropdownMenuContent, DropdownMenuTrigger } from "@radix-ui/react-dropdown-menu";
import { Label } from "@/components/ui/label";
import { db } from "@/lib/db";

const TTSFiltersPage = () => {
  const { data: session, status } = useSession();
  const [loading, setLoading] = useState<boolean>(true)
  const [moreOpen, setMoreOpen] = useState<number>(0)
  const [tag, setTag] = useState("blacklisted")
  const [open, setOpen] = useState<boolean>(false)
  const [userTags, setUserTag] = useState<{ username: string, tag: string }[]>([])
  const router = useRouter();

  const tags = [
    "blacklisted",
    "priority"
  ]

  // Username blacklist
  const usernameFilteredFormSchema = z.object({
    //userId: z.string().trim().min(1),
    username: z.string().trim().min(4).max(25), //.regex(new RegExp("[a-zA-Z0-9][a-zA-Z0-9_]{3, 24}"), "Must be a valid twitch username.")
    tag: z.string().trim()
  });


  const usernameFilteredForm = useForm({
    resolver: zodResolver(usernameFilteredFormSchema),
    defaultValues: {
      //userId: session?.user?.id ?? "",
      username: "",
      tag: ""
    }
  });

  useEffect(() => {
    const fetchData = async () => {
      try {
        const userFiltersData = await axios.get("/api/settings/tts/filter/users");
        setUserTag(userFiltersData.data ?? [])
      } catch (error) {
        console.log("ERROR", error)
      }
    };

    fetchData().catch(console.error);
  }, []);

  const onDelete = () => {
    const username = userTags[Math.log2(moreOpen)].username
    axios.delete("/api/settings/tts/filter/users?username=" + username)
      .then(() => {
        setUserTag(userTags.filter((u) => u.username != username))
      }).catch((e) => console.error(e))
  }

  const isLoading = usernameFilteredForm.formState.isSubmitting;

  const onAdd = (values: z.infer<typeof usernameFilteredFormSchema>) => {
    try {
      values.tag = tag
      axios.post("/api/settings/tts/filter/users", values)
        .then((d) => {
          userTags.push({ username: values.username, tag: tag })
          setUserTag(userTags)

          usernameFilteredForm.reset();
          router.refresh();
        })
    } catch (error) {
      console.log("[TTS/FILTERS/USER]", error);
      return;
    }
  }

  return (
    <div>
      <div className="text-2xl text-center pt-[50px]">TTS Filters</div>
      <div className="px-10 py-10 w-full h-full flex-grow inset-y-1/2">
        <div>
          {userTags.map((user, index) => (
            <div className="flex w-full items-start justify-between rounded-md border px-2 py-3">
              <p className="text-base font-medium">
                <span className="mr-2 rounded-lg bg-primary px-2 py-1 text-xs text-primary-foreground">
                  {user.tag}
                </span>
                <span className="text-white">{user.username}</span>
              </p>
              <DropdownMenu open={(moreOpen & (1 << index)) > 0} onOpenChange={() => setMoreOpen((v) => v ^ (1 << index))}>
                <DropdownMenuTrigger asChild>
                  <Button variant="ghost" size="sm">
                    <MoreHorizontal />
                  </Button>
                </DropdownMenuTrigger>
                <DropdownMenuContent align="end" className="w-[200px]">
                  <DropdownMenuLabel>Actions</DropdownMenuLabel>
                  <DropdownMenuGroup>
                    <DropdownMenuSub>
                      <DropdownMenuSubTrigger>
                        <Tags className="mr-2 h-4 w-4" />
                        Apply label
                      </DropdownMenuSubTrigger>
                      <DropdownMenuSubContent className="p-0">
                        <Command>
                          <CommandInput
                            placeholder="Filter label..."
                            autoFocus={true}
                          />
                          <CommandList>
                            <CommandEmpty>No label found.</CommandEmpty>
                            <CommandGroup>
                              {tags.map((tag) => (
                                <CommandItem
                                  key={tag}
                                  value={tag}
                                  onSelect={(value) => {
                                    userTags[index].tag = value
                                    setUserTag(userTags)
                                    setMoreOpen(0)
                                  }}
                                >
                                  {tag}
                                </CommandItem>
                              ))}
                            </CommandGroup>
                          </CommandList>
                        </Command>
                      </DropdownMenuSubContent>
                    </DropdownMenuSub>
                    <DropdownMenuSeparator />
                    <DropdownMenuItem onClick={onDelete} className="text-red-600">
                      <Trash className="mr-2 h-4 w-4" />
                      Delete
                    </DropdownMenuItem>
                  </DropdownMenuGroup>
                </DropdownMenuContent>
              </DropdownMenu>
            </div>
          ))}
          <Form {...usernameFilteredForm}>
            <form onSubmit={usernameFilteredForm.handleSubmit(onAdd)}>
              <div className="flex w-full items-start justify-between rounded-md border px-4 py-3">
                <Label className="mr-2 rounded-lg bg-primary px-2 py-1 text-xs text-primary-foreground">
                  {tag}
                </Label>
                <FormField
                  control={usernameFilteredForm.control}
                  name="username"
                  render={({ field }) => (
                    <FormItem className="flex-grow">
                      <FormControl>
                        <Input id="username" placeholder="Enter a twitch username" {...field} />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <Button variant="ghost" size="sm" type="submit">
                  <Plus />
                </Button>
                <DropdownMenu open={open} onOpenChange={setOpen}>
                  <DropdownMenuTrigger asChild>
                    <Button variant="ghost" size="sm" {...usernameFilteredForm}>
                      <MoreHorizontal />
                    </Button>
                  </DropdownMenuTrigger>
                  <DropdownMenuContent align="end" className="w-[200px]">
                    <DropdownMenuLabel>Actions</DropdownMenuLabel>
                    <DropdownMenuGroup>
                      <DropdownMenuSub>
                        <DropdownMenuSubTrigger>
                          <Tags className="mr-2 h-4 w-4" />
                          Apply label
                        </DropdownMenuSubTrigger>
                        <DropdownMenuSubContent className="p-0">
                          <Command>
                            <CommandInput
                              placeholder="Filter label..."
                              autoFocus={true}
                            />
                            <CommandList>
                              <CommandEmpty>No label found.</CommandEmpty>
                              <CommandGroup>
                                {tags.map((tag) => (
                                  <CommandItem
                                    value={tag}
                                    onSelect={(value) => {
                                      setTag(value)
                                      setOpen(false)
                                    }}
                                  >
                                    {tag}
                                  </CommandItem>
                                ))}
                              </CommandGroup>
                            </CommandList>
                          </Command>
                        </DropdownMenuSubContent>
                      </DropdownMenuSub>
                    </DropdownMenuGroup>
                  </DropdownMenuContent>
                </DropdownMenu>
              </div>
            </form>
          </Form>
        </div>
      </div>
    </div>
  );
}

export default TTSFiltersPage;